@@ -23,54 +23,14 @@ pgammamix <- function(x, qmix, d, lower.tail = TRUE,
     control <- get_set_param(control)
 
     ## 1 Define the quantile function of the mixing variable ###################
-    special.mix <- NA
-    qW <- if(is.character(qmix)) { # 'qmix' is a character vector
-              qmix <- match.arg(qmix, choices = c("constant", "inverse.gamma", "pareto"))
-              switch(qmix,
-                     "constant" = {
-                         special.mix <- "constant"
-                         function(u) rep(1, length(u))
-                     },
-                     "inverse.gamma" = {
-                         if(hasArg(df)) {
-                             df <- list(...)$df
-                         } else {
-                             stop("'qmix = \"inverse.gamma\"' requires 'df' to be provided.")
-                         }
-                         ## Still allow df = Inf (normal distribution)
-                         stopifnot(is.numeric(df), length(df) == 1, df > 0)
-                         if(is.finite(df)) {
-                             special.mix <- "inverse.gamma"
-                             df2 <- df / 2
-                             function(u) 1 / qgamma(1 - u, shape = df2, rate = df2)
-                         } else {
-                             special.mix <- "constant"
-                             function(u) rep(1, length(u))
-                         }
-                     },
-                     "pareto"= {
-                         if(hasArg(alpha)) {
-                             alpha <- list(...)$alpha
-                         } else {
-                             stop("'qmix = \"pareto\"' requires 'alpha' to be provided.")
-                         }
-                         special.mix <- "pareto"
-                         function(u) (1-u)^(-1/alpha)
-                     },
-                     stop("Currently unsupported 'qmix'"))
-          } else if(is.list(qmix)) { # 'mix' is a list of the form (<character string>, <parameters>)
-              stopifnot(length(qmix) >= 1, is.character(distr <- qmix[[1]]))
-              qmix. <- paste0("q", distr)
-              if(!existsFunction(qmix.))
-                  stop("No function named '", qmix., "'.")
-              function(u)
-                  do.call(qmix., append(list(u), qmix[-1]))
-          } else if(is.function(qmix)) { # 'mix' is the quantile function F_W^- of F_W
-              function(u)
-                  qmix(u, ...)
-          } else stop("'qmix' must be a character string, list or quantile function.")
+    mix_list      <- get_mix_(qmix = qmix, callingfun = "pgammamix", ... ) 
+    qW            <- mix_list[[1]] # function(u)
+    special.mix   <- mix_list[[2]]
+    
     ## Build result object
     pres <- rep(0, n) # n-vector of results
+    abserror <- rep(0, n)
+    relerror <- rep(0, n) 
     notNA <- which(!is.na(x))
     pres[!notNA] <- NA
     x <- x[notNA, drop = FALSE] # non-missing data (rows)
@@ -83,7 +43,7 @@ pgammamix <- function(x, qmix, d, lower.tail = TRUE,
             pres[notNA] <- switch(special.mix,
                                   "inverse.gamma" = {
                                       ## D^2 ~ d* F(d, nu)
-                                      pf(x/d, df1 = d, df2 = df,
+                                      pf(x/d, df1 = d, df2 = mix_list$param,
                                          lower.tail = lower.tail)
                                   },
                                   "constant" = {
@@ -92,7 +52,8 @@ pgammamix <- function(x, qmix, d, lower.tail = TRUE,
                                              lower.tail = lower.tail)
                                   })
             ## Return in those cases
-            attr(pres, "error")   <- rep(0, n)
+            attr(pres, "abs. error") <- rep(0, n)
+            attr(pres, "rel. error") <- rep(0, n)
             attr(pres, "numiter") <- numiter
             return(pres)
         }
@@ -113,10 +74,9 @@ pgammamix <- function(x, qmix, d, lower.tail = TRUE,
         tol <- control$pgammamix.reltol
         do.reltol <- TRUE
     }
-    ## Store seed if 'sobol' is used to get the same shifts later:
+    ## Store seed if 'sobol' is used to get the same shifts later
     if(control$method == "sobol") {
-        if(!exists(".Random.seed")) runif(1) # dummy to generate .Random.seed
-        seed <- .Random.seed # need to reset to the seed later if a Sobol sequence is being used
+       seeds_ <- sample(1:(1e3*B), B) # B seeds for 'sobol()'
     }
     ## Additional variables needed if the increment chosen is "dblng"
     if(dblng) {
@@ -137,9 +97,7 @@ pgammamix <- function(x, qmix, d, lower.tail = TRUE,
     while(max.error > tol && numiter < control$max.iter.rqmc &&
           total.fun.evals < control$fun.eval[2])
     {
-        ## Reset seed to have the same shifts in sobol(...)
-        if(control$method == "sobol" && numiter > 0)
-            .Random.seed <<- seed # reset seed to have the same shifts in sobol(...)
+
         for(b in 1:B) {
 
             ## 2.1 Get the point set
@@ -147,11 +105,13 @@ pgammamix <- function(x, qmix, d, lower.tail = TRUE,
                         "sobol" = {
                             if(dblng) {
                                 qrng::sobol(n = current.n, d = 1,
-                                            randomize = TRUE,
+                                            randomize = "digital.shift",
+                                            seed = seeds_[b],
                                             skip = (useskip * current.n))
                             } else {
                                 qrng::sobol(n = current.n, d = 1,
-                                            randomize = TRUE,
+                                            randomize = "digital.shift",
+                                            seed = seeds_[b],
                                             skip = (numiter * current.n))
                             }
                         },
@@ -196,24 +156,33 @@ pgammamix <- function(x, qmix, d, lower.tail = TRUE,
                 current.n <- 2 * current.n
             }
         }
-        ## Total number of function evaluations:
-        total.fun.evals <- total.fun.evals + B * current.n
-        numiter <- numiter + 1
-        ## Update error. The following is slightly faster than 'apply(..., 2, var)'
-        pres[notNA] <- .colMeans(rqmc.estimates, B, n , 0)
-        vars <- .colMeans((rqmc.estimates - rep(pres, each = B))^2, B, n, 0)
-        errors <- if(!do.reltol) {
-                      sqrt(vars)*CI.factor.sqrt.B
-                  } else {
-                      sqrt(vars)/pres*CI.factor.sqrt.B
-                  }
-        max.error <- max(errors)
+       ## Total number of function evaluations
+       total.fun.evals <- total.fun.evals + B * current.n
+       numiter <- numiter + 1
+       ## Update error. The following is slightly faster than 'apply(..., 2, var)'
+       pres[notNA] <- .colMeans(rqmc.estimates, B, n , 0)
+       vars <- .colMeans((rqmc.estimates - rep(pres, each = B))^2, B, n, 0)
+       error <- if(!do.reltol) {
+          sqrt(vars)*CI.factor.sqrt.B
+       } else {
+          sqrt(vars)/pres*CI.factor.sqrt.B
+       }
+       max.error <- max(error)
     } # while()
-    if(verbose && max.error > tol)
-        warning("Tolerance not reached for all inputs; consider increasing 'max.iter.rqmc' in the 'control' argument.")
-
+    if(verbose & max.error > tol)
+       warning("Tolerance not reached for all inputs; consider increasing 'max.iter.rqmc' in the 'control' argument.")
+    
+    ## Compute absolute and relative errors (once here at the end)
+    abserror[notNA] <- if(do.reltol){
+       relerror[notNA] <- error
+       relerror[notNA] * pres[notNA] 
+    } else { # error is absolute error
+       relerror[notNA] <- error / pres[notNA] 
+       error 
+    }
     ## Return
-    attr(pres, "error")   <- errors
+    attr(pres, "abs. error") <- abserror
+    attr(pres, "rel. error") <- relerror
     attr(pres, "numiter") <- numiter
     pres
 }
@@ -251,19 +220,10 @@ qgammamix <- function(u, qmix, d, control = list(), verbose = TRUE, q.only = TRU
 ##' @param ... n-vector or realizations
 ##' @return n-vector of samples of W*X where W~qmix/rmix and X~chi^2_d = Gamma(shape = d/2, scale = 2)
 ##' @author Marius Hofert and Erik Hintz
-##' @note - For the Student t distribution, W ~ df/rchisq(n, df = df) but
-##'         rchisq() simply calls rgamma(); see ./src/nmath/rchisq.c
-##'         => W ~ 1/rgamma(n, shape = df/2, rate = df/2)
-##'       - For a generalized inverse Gaussian distribution one could use:
-##'         + "Runuran": faster if n large and parameters fixed; based on density
-##'         + "GIGrvg":  faster if n small and often called with several parameters
-##'         see examples of 'GIGrvg' for both methods
-##'       - user friendly wrappers are provided in 'rnvmix()' and 'rgammamix()'
-rgammamix <- function(n, rmix = NULL, qmix = NULL, d,
-                      method = c("PRNG", "sobol", "ghalton"), skip = 0, ...)
+rgammamix <- function(n, rmix, qmix, d, method = c("PRNG", "sobol", "ghalton"), 
+                      skip = 0, ...)
     rnvmix_(n, rmix = rmix, qmix = qmix, loc = rep(0, d), scale = diag(d),
-                  factor = diag(d), method = method, skip = skip, which = "maha2",
-                  ...)
+            factor = diag(d), method = method, skip = skip, which = "maha2", ...)
 
 
 ### dgammamix() ################################################################
@@ -288,91 +248,42 @@ dgammamix <- function(x, qmix, d, control = list(), verbose = TRUE, log = FALSE,
     verbose <- as.logical(verbose)
     ## Deal with algorithm parameters, see also get_set_param()
     control <- get_set_param(control)
-
     ## Build result object (log-density)
     lres  <- rep(-Inf, n) # n-vector of results
+    abserror <- rep(NA, n)
+    relerror <- rep(NA, n) 
     notNA <- which(!is.na(x))
     lres[!notNA] <- NA
     x <- x[notNA, drop = FALSE] # non-missing data (rows)
-
-    ## 1 Define the quantile function of the mixing variable ###################
-
-    ## If 'mix' is "constant", "inverse.gamma" or "pareto", we use the analytical formulas
-    special.mix <- NA
-    qW <- if(is.character(qmix)) { # 'qmix' is a character vector
-              qmix <- match.arg(qmix, choices = c("constant", "inverse.gamma", "pareto"))
-              switch(qmix,
-                     "constant" = {
-                         special.mix <- "constant"
-                         function(u) rep(1, length(u))
-                     },
-                     "inverse.gamma" = {
-                         if(hasArg(df)) {
-                             df <- list(...)$df
-                         } else {
-                             stop("'qmix = \"inverse.gamma\"' requires 'df' to be provided.")
-                         }
-                         ## Still allow df = Inf (normal distribution)
-                         stopifnot(is.numeric(df), length(df) == 1, df > 0)
-                         if(is.finite(df)) {
-                             special.mix <- "inverse.gamma"
-                             df2 <- df / 2
-                             mean.sqrt.mix <- sqrt(df) * gamma(df2) /
-                                 (sqrt(2) * gamma((df+1)/2)) # used for preconditioning
-                             function(u) 1 / qgamma(1 - u, shape = df2, rate = df2)
-                         } else {
-                             special.mix <- "constant"
-                             mean.sqrt.mix <- 1 # used for preconditioning
-                             function(u) rep(1, length(u))
-                         }
-                     },
-                     "pareto"= {
-                         if(hasArg(alpha)) {
-                             alpha <- list(...)$alpha
-                         } else {
-                             stop("'qmix = \"pareto\"' requires 'alpha' to be provided.")
-                         }
-                         special.mix <- "pareto"
-                         mean.sqrt.mix <- if(alpha > 0.5) alpha/(alpha-0.5) else NULL
-                         function(u) (1-u)^(-1/alpha)
-                     },
-                     stop("Currently unsupported 'qmix'"))
-          } else if(is.list(qmix)) { # 'mix' is a list of the form (<character string>, <parameters>)
-              stopifnot(length(qmix) >= 1, is.character(distr <- qmix[[1]]))
-              qmix. <- paste0("q", distr)
-              if(!existsFunction(qmix.))
-                  stop("No function named '", qmix., "'.")
-              function(u)
-                  do.call(qmix., append(list(u), qmix[-1]))
-          } else if(is.function(qmix)) { # 'mix' is the quantile function F_W^- of F_W
-              function(u)
-                  qmix(u, ...)
-          } else stop("'qmix' must be a character string, list or quantile function.")
-
+    ## Define the quantile function of the mixing variable 
+    if(!hasArg(qmix)) qmix <- NULL # needed for 'get_mix_()'
+    mix_list      <- get_mix_(qmix = qmix, callingfun = "dgammamix", ... ) 
+    qW            <- mix_list[[1]] # function(u)
+    special.mix   <- mix_list[[2]] # NA or string 
     ## Counter
     numiter <- 0 # initialize counter (0 for 'inv.gam' and 'is.const.mix')
-    ## Deal with the different distributions
-
     ## Deal with special distributions
     if(!is.na(special.mix)) {
         if(!(special.mix == "pareto")) {
-            ## Only for "inverse.gamma" and "constant" do we have analytical forms:
+            ## Only for "inverse.gamma" and "constant" do we have analytical forms
             lres[notNA] <- switch(special.mix,
                                   "inverse.gamma" = {
                                       ## D^2 ~ d* F(d, nu)
-                                      df(x/d, df1 = d, df2 = df, log = TRUE) - log(d)
+                                      df(x/d, df1 = d, df2 = mix_list$param, 
+                                         log = TRUE) - log(d)
                                   },
                                   "constant" = {
                                       ## D^2 ~ chi^2_d = Gamma(shape = d/2, scale = 2)
                                       dgamma(x, shape = d/2, scale = 2, log = TRUE)
                                   })
             ## Return in those cases
-            attr(lres, "error")   <- rep(0, n)
+            attr(lres, "abs. error") <- rep(0, n)
+            attr(lres, "rel. error") <- rep(0, n)
             attr(lres, "numiter") <- numiter
             if(log) return(lres) else return(exp(lres))
         }
     }
-    ## General case of a multivariate normal variance mixture (RQMC)
+    ## General case of a multivariate normal variance mixture (=> RQMC procedure)
     ## Prepare inputs for 'densmix_()'
     ## Sort input 'x' increasingly and store ordering for later
     ordering.x <- order(x)
@@ -380,21 +291,26 @@ dgammamix <- function(x, qmix, d, control = list(), verbose = TRUE, log = FALSE,
     ## Define log-constant for the integration
     lconst <- -lgamma(d/2) - d/2 * log(2) + (d/2-1)*log(x.ordered)
     ## Call 'densmix_()' (which itself calls C-Code and handles warnings):
-    ests <- densmix_(qW, maha2.2 = x.ordered/2, lconst = lconst,
+    est.list <- densmix_(qW, maha2.2 = x.ordered/2, lconst = lconst,
                              d = d, control = control, verbose = verbose)
     ## Grab results, correct 'error' and 'lres' if 'log = FALSE'
-    lres[notNA] <- ests$ldensities[order(ordering.x)]
-    error <- if(log) {
-                 ests$error[order(ordering.x)]
-             } else {
-                 lres <- exp(lres)
-                 ests$error[order(ordering.x)]*pmax(lres[notNA], 1)
-             }
-    numiter <- ests$numiter
-
+    ldens <- est.list$ldensities[order(ordering.x)]
+    abserror[notNA] <- est.list$abserror[order(ordering.x)]
+    relerror[notNA] <- est.list$relerror[order(ordering.x)]
+    ## Correct results and error if 'log = FALSE'
+    if(!log){
+       ldens <- exp(ldens)
+       ## CI for mu: exp(logmu_hat +/- abserr(logmu_hat))) = (lower, upper)
+       ## => compute max. error on mu_hat as max( (upper - mu), (mu - lower) ) 
+       relerror[notNA] <- max( (exp(abserror[notNA]) - 1), (1 - exp(-abserror[notNA])) )
+       abserror[notNA] <- ldens * relerror[notNA] # ldens already exponentiated 
+    }
+    lres[notNA] <- ldens 
+    
     ## Return
     ## Note that 'lres' was exponentiated already if necessary.
-    attr(lres, "error")   <- error # these are absolute errors, no matter what!
-    attr(lres, "numiter") <- numiter
+    attr(lres, "abs. error") <- abserror
+    attr(lres, "rel. error") <- relerror
+    attr(lres, "numiter") <- est.list$numiter
     lres
 }
